/*
 * 
 *  This file is part of Sourceprojects.org Lycia Copyright 2010 noctarius
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 */
package org.sourceprojects.lycia.fluent;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import org.sourceprojects.lycia.Generator;
import org.sourceprojects.lycia.GeneratorFactory;
import org.sourceprojects.lycia.LyciaConfigurator;
import org.sourceprojects.lycia.LyciaParser;
import org.sourceprojects.lycia.ProgressListener;
import org.sourceprojects.lycia.ResourceResolver;
import org.sourceprojects.lycia.GenerationStrategy;
import org.sourceprojects.lycia.exceptions.LyciaInitializationException;
import org.sourceprojects.lycia.io.LyciaReader;
import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;

@SuppressWarnings("unchecked")
class ConfigurationBuilderHelper {
	private ConfigurationBuilderHelper() {
	}

	static <B> ConfigurationBuilder<B> createValidatingSchema(
			final boolean validatingSchema) {

		return new AbstractConfigurationBuilder<B>() {
			@Override
			public void configure(final LyciaParser<B> xmlParser) {
				xmlParser.setValidatingSchema(validatingSchema);
			}
		};
	}

	static <B> ConfigurationBuilder<B> createGenerationStrategy(
			final GenerationStrategy generationStrategy,
			final Class<? extends Generator> generatorClass) {

		if (generationStrategy == null) {
			throw new IllegalArgumentException(
					"GenerationStrategy cannot be null");
		}

		if (generationStrategy == GenerationStrategy.Customized
				&& generatorClass == null) {
			throw new IllegalArgumentException(
					"When using GenerationStrategy.Customized Generator class cannot be null");
		}

		return new AbstractConfigurationBuilder<B>() {
			@Override
			public int getPriority() {
				return Integer.MIN_VALUE + 1;
			}

			@Override
			public void configure(final LyciaParser<B> xmlParser) {
				final LyciaConfigurator<B> configurator = xmlParser
						.getConfigurator();

				configurator.setGenerationStrategy(generationStrategy);
				if (generationStrategy == GenerationStrategy.Customized) {
					configurator.getGeneratorFactory().setCustomGeneratorClass(
							(Class<? extends Generator<B>>) generatorClass);
				}
			}
		};
	}

	static <B> ConfigurationBuilder<B> createXmlParserConfigurator(
			final Class<? extends LyciaConfigurator> configuratorClass) {

		if (configuratorClass == null) {
			throw new IllegalArgumentException(
					"XmlParserConfigurator class cannot be null");
		}

		return new AbstractConfigurationBuilder<B>() {
			@Override
			public int getPriority() {
				return Integer.MIN_VALUE;
			}

			@Override
			public void configure(final LyciaParser<B> xmlParser) {
				try {
					Constructor<? extends LyciaConfigurator<B>> constructor;

					final Class<? extends LyciaConfigurator<B>> clazz = (Class<? extends LyciaConfigurator<B>>) configuratorClass;
					constructor = clazz.getConstructor(LyciaParser.class,
							LyciaConfigurator.class);

					final LyciaConfigurator<B> configurator = constructor
							.newInstance(xmlParser, xmlParser.getConfigurator());

					xmlParser.setConfigurator(configurator);
				} catch (final InvocationTargetException e) {
					throw new LyciaInitializationException(e);
				} catch (final IllegalArgumentException e) {
					throw new LyciaInitializationException(e);
				} catch (final InstantiationException e) {
					throw new LyciaInitializationException(e);
				} catch (final IllegalAccessException e) {
					throw new LyciaInitializationException(e);
				} catch (final SecurityException e) {
					throw new LyciaInitializationException(e);
				} catch (final NoSuchMethodException e) {
					throw new LyciaInitializationException(e);
				}
			}
		};
	}

	static <B> ConfigurationBuilder<B> createContentHandler(
			final ContentHandler... contentHandlers) {

		return new AbstractConfigurationBuilder<B>() {
			@Override
			public void configure(final LyciaParser<B> xmlParser) {
				for (final ContentHandler contentHandler : contentHandlers) {
					xmlParser.getConfigurator().addContentHandler(
							contentHandler);
				}
			}
		};
	}

	static <B> ConfigurationBuilder<B> createDoNothingConfigurator() {
		return new AbstractConfigurationBuilder<B>() {
			@Override
			public void configure(final LyciaParser<B> xmlParser) {
				// just do nothing
			}
		};
	}

	static <B> ConfigurationBuilder<B> createContextObjectConfigurator(
			final Class<?> contextObjectClass) {

		try {

			final B contextObject = (B) contextObjectClass.newInstance();

			return new AbstractConfigurationBuilder<B>() {
				@Override
				public void configure(final LyciaParser<B> xmlParser) {
					xmlParser.getParserContext()
							.setContextObject(contextObject);
				}
			};
		} catch (final InstantiationException e) {
			throw new LyciaInitializationException(e);
		} catch (final IllegalAccessException e) {
			throw new LyciaInitializationException(e);
		}
	}

	static <B> ConfigurationBuilder<B> createContextObjectConfigurator(
			final B contextObject) {

		return new AbstractConfigurationBuilder<B>() {
			@Override
			public void configure(final LyciaParser<B> xmlParser) {
				xmlParser.getParserContext().setContextObject(contextObject);
			}
		};
	}

	static <B> ConfigurationBuilder<B> createResourceResolverConfigurator(
			final ResourceResolver resourceResolver) {

		return new AbstractConfigurationBuilder<B>() {
			@Override
			public void configure(final LyciaParser<B> xmlParser) {
				xmlParser.setResourceResolver(resourceResolver);
			}
		};
	}

	static <B> ConfigurationBuilder<B> createErrorHandlerConfigurator(
			final ErrorHandler errorHandler) {

		return new AbstractConfigurationBuilder<B>() {
			@Override
			public void configure(final LyciaParser<B> xmlParser) {
				xmlParser.setErrorHandler(errorHandler);
			}
		};
	}

	static <B> ConfigurationBuilder<B> createProgressListenerConfigurator(
			final ProgressListener... progressListeners) {

		return new AbstractConfigurationBuilder<B>() {
			@Override
			public void configure(final LyciaParser<B> xmlParser) {
				for (final ProgressListener progressListener : progressListeners) {
					xmlParser.addProgressListener(progressListener);
				}
			}
		};
	}

	static <B> ConfigurationBuilder<B> createLyciaReaderConfigurator(
			final Class<LyciaReader<B>> lyciaReaderClass) {

		return new AbstractConfigurationBuilder<B>() {
			@Override
			public void configure(final LyciaParser<B> xmlParser) {
				xmlParser.setReader(lyciaReaderClass);
			}
		};
	}

	static <B> ConfigurationBuilder<B> createCacheExpirationTimeConfigurator(
			final long expirationTime) {

		return new AbstractConfigurationBuilder<B>() {
			@Override
			public void configure(final LyciaParser<B> xmlParser) {
				xmlParser.setCacheExpirationTime(expirationTime);
			}
		};
	}

	static <B> ConfigurationBuilder<B> createGeneratorFactoryConfigurator(
			final GeneratorFactory<B> generatorFactory) {

		return new AbstractConfigurationBuilder<B>() {

			@Override
			public void configure(final LyciaParser<B> xmlParser) {
				xmlParser.getConfigurator().setGeneratorFactory(
						generatorFactory);
			}
		};
	}

	static abstract class AbstractConfigurationBuilder<B> implements
			ConfigurationBuilder<B> {

		public int getPriority() {
			return 0;
		}
	}
}
